searchData={"items":[{"type":"module","title":"OnePiece.Commanded","doc":"Extend `Commanded` package. A swiss army knife for applications following Domain-Driven Design (DDD), Event Sourcing\n(ES), and Command and Query Responsibility Segregation (CQRS).","ref":"OnePiece.Commanded.html"},{"type":"function","title":"OnePiece.Commanded.cast_to/3","doc":"","ref":"OnePiece.Commanded.html#cast_to/3"},{"type":"function","title":"OnePiece.Commanded.ignore_error/2","doc":"","ref":"OnePiece.Commanded.html#ignore_error/2"},{"type":"function","title":"OnePiece.Commanded.increase_failure_counter/1","doc":"","ref":"OnePiece.Commanded.html#increase_failure_counter/1"},{"type":"function","title":"OnePiece.Commanded.skip_or_retry/2","doc":"","ref":"OnePiece.Commanded.html#skip_or_retry/2"},{"type":"function","title":"OnePiece.Commanded.skip_or_retry/3","doc":"","ref":"OnePiece.Commanded.html#skip_or_retry/3"},{"type":"function","title":"OnePiece.Commanded.tracing_from_metadata/1","doc":"","ref":"OnePiece.Commanded.html#tracing_from_metadata/1"},{"type":"function","title":"OnePiece.Commanded.tracing_from_metadata/2","doc":"","ref":"OnePiece.Commanded.html#tracing_from_metadata/2"},{"type":"module","title":"OnePiece.Commanded.Aggregate.StatelessLifespan","doc":"Stops the aggregate after a command, event or error.","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html"},{"type":"function","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_command/1","doc":"Stops the aggregate after a command.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_command(%MyCommandOne{})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_command/1"},{"type":"function","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_error/1","doc":"Stops the aggregate after an error.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_error({:error, :something_happened})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_error/1"},{"type":"function","title":"OnePiece.Commanded.Aggregate.StatelessLifespan.after_event/1","doc":"Stops the aggregate after an event.\n    iex> OnePiece.Commanded.Aggregate.StatelessLifespan.after_event(%DepositAccountOpened{})\n    :stop","ref":"OnePiece.Commanded.Aggregate.StatelessLifespan.html#after_event/1"},{"type":"module","title":"OnePiece.Commanded.CommandHandler","doc":"Defines a module as a \"Command Handler\". For more information about commands, please read the following:\n\n- [CQRS pattern](https://docs.microsoft.com/en-us/azure/architecture/patterns/cqrs)","ref":"OnePiece.Commanded.CommandHandler.html"},{"type":"macro","title":"OnePiece.Commanded.CommandHandler.__using__/1","doc":"Convert the module into a `Commanded.Commands.Handler`.\n\nUse `Commanded.Aggregate.Multi` to generate multiple events from\na single command. This can be useful when you want to emit multiple events\nthat depend upon the aggregate state being updated.","ref":"OnePiece.Commanded.CommandHandler.html#__using__/1"},{"type":"macro","title":"Usage - OnePiece.Commanded.CommandHandler.__using__/1","doc":"defmodule MyCommandHandler do\n      use OnePiece.Commanded.CommandHandler\n\n      @impl Commanded.Commands.Handler\n      def handle(account, command) do\n        account\n        |> Multi.new()\n        |> Multi.execute(&withdraw_money(&1, command.amount))\n        |> Multi.execute(&check_balance/1)\n      end\n\n      defp withdraw_money(account, amount) do\n        # ...\n      end\n\n      defp check_balance(account, amount) do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.CommandHandler.html#__using__/1-usage"},{"type":"module","title":"OnePiece.Commanded.CommandRouter","doc":"Command routing macro to allow configuration of each command to its command handler.\n\nPlease read more about `Commanded.Commands.Router`.","ref":"OnePiece.Commanded.CommandRouter.html"},{"type":"macro","title":"OnePiece.Commanded.CommandRouter.__using__/1","doc":"An extension of `Commanded.Commands.Router` that imports extra macros.","ref":"OnePiece.Commanded.CommandRouter.html#__using__/1"},{"type":"macro","title":"Usage - OnePiece.Commanded.CommandRouter.__using__/1","doc":"defmodule Router do\n      use OnePiece.Commanded.CommandRouter\n    end","ref":"OnePiece.Commanded.CommandRouter.html#__using__/1-usage"},{"type":"macro","title":"OnePiece.Commanded.CommandRouter.identify_aggregate/1","doc":"Identify a given aggregate.","ref":"OnePiece.Commanded.CommandRouter.html#identify_aggregate/1"},{"type":"macro","title":"Example - OnePiece.Commanded.CommandRouter.identify_aggregate/1","doc":"defmodule BankAccount do\n      use OnePiece.Commanded.Aggregate\n        identifier: :uuid,\n        identity_prefix: \"bank-account-\"\n\n      embedded_schema do\n        # ...\n      end\n    end\n\n    defmodule Router do\n      use OnePiece.Commanded.CommandRouter\n      identify_aggregate BankAccount\n    end","ref":"OnePiece.Commanded.CommandRouter.html#identify_aggregate/1-example"},{"type":"macro","title":"OnePiece.Commanded.CommandRouter.register_transaction_script/2","doc":"Configure the command to be dispatched to the same command module.","ref":"OnePiece.Commanded.CommandRouter.html#register_transaction_script/2"},{"type":"macro","title":"Example - OnePiece.Commanded.CommandRouter.register_transaction_script/2","doc":"defmodule Router do\n      use OnePiece.Commanded.CommandRouter\n      register_transaction_script OpenBankAccount,\n        aggregate: BankAccount\n    end\n\n    defmodule Router do\n      use OnePiece.Commanded.CommandRouter\n      # the aggregate module is `OpenBankAccount.Aggregate`\n      register_transaction_script OpenBankAccount\n    end\n\n\nSimilar to `Commanded.Commands.Router.dispatch/2` except that,\n\n1. It uses the command module as the command handler as well.\n2. It uses the command module's aggregate identifier configuration.\n3. When no lifespan module is provided, it uses `OnePiece.Commanded.Aggregate.StatelessLifespan`. as the lifespan\n   module to avoid OOM errors. Reliability is more important than performance.\n4. When no aggregate module is provided, it uses the command module name suffixed concat with `Aggregate` as the\n   aggregate module.\n   As an example, if the command module is `OpenBankAccount`, then the aggregate module is\n   `OpenBankAccount.Aggregate`.\n5. Drops some options to avoid conflicts with the command module configuration.","ref":"OnePiece.Commanded.CommandRouter.html#register_transaction_script/2-example"},{"type":"macro","title":"About the Transaction Script naming - OnePiece.Commanded.CommandRouter.register_transaction_script/2","doc":"The naming comes from [Transaction Script from Martin Fowler](https://martinfowler.com/eaaCatalog/transactionScript.html),\nalthough it is not exactly the same since the original name is about database transactions, the key part is the\nfollowing sentence:\n\n> A Transaction Script organizes all this logic primarily as a single procedure.\n\nThis is the idea behind the `register_transaction_script/2` macro is to have the strongest cohesion possible for a\ngiven Use Case (Command in this case).\nDiscouraging patterns around a single Command Handler that handles multiple commands, that over time, can become a\na [God Module](https://en.wikipedia.org/wiki/God_object),\n\nGod Modules that becomes difficult to maintain and understand. Having to come up with weird names for functions, scroll\npass hundreds of lines of code to find the right piece of code to change, or bike-shedding around locations of the\ncode, to the point that some developers will annotate with comments to indicate Labels for IDEs to jump to the right\nplace, ect.","ref":"OnePiece.Commanded.CommandRouter.html#register_transaction_script/2-about-the-transaction-script-naming"},{"type":"macro","title":"Why to have an Aggregate per Command? - OnePiece.Commanded.CommandRouter.register_transaction_script/2","doc":"The needs for an given Aggregate is driven by the command handler that requires such \"state\", it is not the other way\naround. The same problems with the God Module can happen with the God Aggregate.\n\nIn practices, most of the `apply/2` functions will be a simple functions, barely anything passed basic permutations.\nSince the Aggregate state needs is driven by the command handler, it makes sense to have a 1:1 relationship between\nthe command and the aggregate.\n\nThe only dependency between the command handlers are the events, not the \"state\" of the aggregate. The existence of\na given event may be due to another command handler, that is OK, a set of Command Handlers acts upon the same\nfacts/events, the past is immutable, the past is the same for everyone, not the state of the aggregate.\nThat is one reason why when you are doing testing you speak in terms of \"given previous events\" and not\n\"given previous state\".\n\nThe \"Aggregate\", lets remove that word from our vocabulary, the \"Stream\" acts as a consistency boundary enforcing the\nlinearization and concurrency tolerance thru Optimistic Concurrency Control (OCC). That is all! It is about agreeing\non acting upon the same facts/events, and how its affects the future.\n\nBeing said, be pragmatic.","ref":"OnePiece.Commanded.CommandRouter.html#register_transaction_script/2-why-to-have-an-aggregate-per-command"},{"type":"macro","title":"FAQ - OnePiece.Commanded.CommandRouter.register_transaction_script/2","doc":"Q: Should I have a `command.ex` and `command/aggregate.ex` files?\nA: Yes, but it is discouraged, use the same `command.ex` file to put the\ncommand and the aggregate. As we said before, high cohesion of the code is\nthe goal, not the number of files, the more files you have, the more entropy\nyou have to deal with, and since the aggregate is driven by the command\nhandler, you are most likely to modify the command and the aggregate at the\nsame time, or in the best case, ignore the aggregate module.\n\nHere is an example file structure:\n\n    defmodule BankAccount.OpenBankAccount.Aggregate do\n      use OnePiece.Commanded.Aggregate, identifier: :uuid\n      embedded_schema do\n        # ...\n      end\n    end\n\n    defmodule BankAccount.OpenBankAccount do\n      use OnePiece.Commanded.CommandHandler\n      use OnePiece.Commanded.Command,\n        aggregate_identifier: :uuid,\n        identity_prefix: \"bank-account-\"\n\n      alias BankAccount.{\n        BankAccountOpened,\n        OpenBankAccount\n      }\n\n      embedded_schema do\n      end\n\n      def handle(%OpenBankAccount.Aggregate{} = aggregate, %OpenBankAccount{} = command) do\n        # ...\n      end\n    end\n\n\n\nQ: Am I allowed to reuse the same aggregate for multiple commands?\nA: Yes, you can, but it is discouraged, since it can lead to the God Aggregate problem.\n\nQ: Am I allowed to reuse code between aggregates?\nA: Yes, you can, but it is discouraged, a simple copy+paste could safe you from a lot of headaches in the future. Be\nmindful of the [Rule of Three](https://en.wikipedia.org/wiki/Rule_of_three_(computer_programming)). Use the\n`MyApp.[Stream Name].[Stream Name]` module as a place to put common code between aggregates.","ref":"OnePiece.Commanded.CommandRouter.html#register_transaction_script/2-faq"},{"type":"module","title":"OnePiece.Commanded.EventStore.JsonbSerializer","doc":"A JSONB serializer based on events defined by `OnePiece.Commanded.Event`.\n\nIt uses `OnePiece.Commanded.Event` to cast the event to the correct type, by proxying, using `Ecto.Schema`s and\n`Ecto.Changeset`s.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html"},{"type":"module","title":"Configuring - OnePiece.Commanded.EventStore.JsonbSerializer","doc":"To use this serializer, add it to your `config.exs`:\n\n    config :my_app, MyApp.EventStore,\n      serializer: OnePiece.Commanded.EventStore.JsonbSerializer,\n      types: EventStore.PostgresTypes","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#module-configuring"},{"type":"function","title":"OnePiece.Commanded.EventStore.JsonbSerializer.deserialize/2","doc":"Deserialize given JSON binary data to the expected type.\n\nIt is already a map since `EventStore.PostgresTypes` will take care of the deserialization. Then, it will use\n`OnePiece.Commanded.Event` to cast the event to the correct type.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#deserialize/2"},{"type":"function","title":"OnePiece.Commanded.EventStore.JsonbSerializer.serialize/1","doc":"Serialize given term to JSON binary data.\n\nIt is just a passthrough, since `EventStore.PostgresTypes` will take care of the serialization.","ref":"OnePiece.Commanded.EventStore.JsonbSerializer.html#serialize/1"},{"type":"module","title":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer","doc":"A JSONB serializer using `Protobuf.JSON`.","ref":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.html"},{"type":"module","title":"Configuring - OnePiece.Commanded.EventStore.ProtobufJsonbSerializer","doc":"To use this serializer, add it to your `config.exs`:\n\n    config :my_app, MyApp.EventStore,\n        serializer: OnePiece.Commanded.EventStore.ProtobufJsonbSerializer,\n        types: EventStore.PostgresTypes","ref":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.html#module-configuring"},{"type":"module","title":"Aggregate Snapshots - OnePiece.Commanded.EventStore.ProtobufJsonbSerializer","doc":"To serialize aggregate snapshots, you need to implement the `OnePiece.Commanded.ProtobufMapper` protocol.","ref":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.html#module-aggregate-snapshots"},{"type":"function","title":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.deserialize/2","doc":"Convert given map to a `Protobuf` struct.","ref":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.html#deserialize/2"},{"type":"function","title":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.serialize/1","doc":"Convert given `Protobuf` struct to a map.","ref":"OnePiece.Commanded.EventStore.ProtobufJsonbSerializer.html#serialize/1"},{"type":"module","title":"OnePiece.Commanded.Helpers","doc":"A Swiss Army Knife Helper Module.","ref":"OnePiece.Commanded.Helpers.html"},{"type":"function","title":"OnePiece.Commanded.Helpers.cast_to/3","doc":"Copy the information from the `source` map into the given `target` map.\n\n    iex> OnePiece.Commanded.Helpers.cast_to(%{}, %{name: \"ubi-wan\", last_name: \"kenobi\"}, [:last_name])\n    %{last_name: \"kenobi\"}","ref":"OnePiece.Commanded.Helpers.html#cast_to/3"},{"type":"type","title":"OnePiece.Commanded.Helpers.commanded_dispatch_response/0","doc":"","ref":"OnePiece.Commanded.Helpers.html#t:commanded_dispatch_response/0"},{"type":"type","title":"OnePiece.Commanded.Helpers.error_context/0","doc":"","ref":"OnePiece.Commanded.Helpers.html#t:error_context/0"},{"type":"function","title":"OnePiece.Commanded.Helpers.generate_uuid/0","doc":"Deprecated, it has the same behavior as `OnePiece.Commanded.Id.new/0`.","ref":"OnePiece.Commanded.Helpers.html#generate_uuid/0"},{"type":"function","title":"OnePiece.Commanded.Helpers.ignore_error/2","doc":"Ignores a specific error from a command dispatch result.\n\nThis function takes a dispatch result and an error term. If the result represents an error that matches the provided\nerror term, the function returns `:ok`. Otherwise, it returns the original result.\n\nThis is useful when an error condition should be treated as a successful operation under specific circumstances.","ref":"OnePiece.Commanded.Helpers.html#ignore_error/2"},{"type":"function","title":"Examples - OnePiece.Commanded.Helpers.ignore_error/2","doc":"iex> OnePiece.Commanded.Helpers.ignore_error({:error, :idempotency_failure}, :idempotency_failure)\n    :ok\n\n    iex> OnePiece.Commanded.Helpers.ignore_error({:error, :something_went_wrong}, :idempotency_failure)\n    {:error, :something_went_wrong}\n\n    iex> OnePiece.Commanded.Helpers.ignore_error(:ok, :idempotency_failure)\n    :ok\n\n    iex> OnePiece.Commanded.Helpers.ignore_error({:ok, %{name: \"Billy\"}}, :idempotency_failure)\n    {:ok, %{name: \"Billy\"}}","ref":"OnePiece.Commanded.Helpers.html#ignore_error/2-examples"},{"type":"function","title":"OnePiece.Commanded.Helpers.increase_failure_counter/1","doc":"Increase the failure counter from `t:Commanded.Event.FailureContext.t/0` context by one.\n\n      iex> OnePiece.Commanded.Helpers.increase_failure_counter(%Commanded.Event.FailureContext{context: %{failures_count: 1}})\n      %{failures_count: 2}","ref":"OnePiece.Commanded.Helpers.html#increase_failure_counter/1"},{"type":"function","title":"OnePiece.Commanded.Helpers.skip_or_retry/2","doc":"Returns `skip` or a `retry` response.\n\nWhen the `c:Commanded.Application.dispatch/1` or `c:Commanded.Application.dispatch/2` returns an `:skip` otherwise,\nreturns a `:retry` response. Useful when you are doing error handling in your `c:Commanded.Event.Handler.error/3`.\n\n    iex> success_dispatch = fn _ -> :ok end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), %{})\n    :skip\n\n    iex> success_dispatch = fn _ -> {:ok, %{}} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), %{})\n    :skip\n\n    iex> failure_dispatch = fn _ -> {:error, :ooops} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(failure_dispatch.(%{}), %{failures: 1})\n    {:retry, %{failures: 1}}","ref":"OnePiece.Commanded.Helpers.html#skip_or_retry/2"},{"type":"function","title":"OnePiece.Commanded.Helpers.skip_or_retry/3","doc":"Returns `skip` or a `retry` response with a given delay.\n\nWhen the `c:Commanded.Application.dispatch/1` or `c:Commanded.Application.dispatch/2` returns an `:skip` otherwise,\nreturns a `:retry` response. Useful when you are doing error handling in your `c:Commanded.Event.Handler.error/3`.\n\n    iex> success_dispatch = fn _ -> :ok end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), 5_000, %{})\n    :skip\n\n    iex> success_dispatch = fn _ -> {:ok, %{}} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(success_dispatch.(%{}), 5_000, %{})\n    :skip\n\n    iex> failure_dispatch = fn _ -> {:error, :ooops} end\n    ...> OnePiece.Commanded.Helpers.skip_or_retry(failure_dispatch.(%{}), 5_000, %{failures: 1})\n    {:retry, 5_000, %{failures: 1}}","ref":"OnePiece.Commanded.Helpers.html#skip_or_retry/3"},{"type":"function","title":"OnePiece.Commanded.Helpers.struct_from/2","doc":"Transforms the given `source` map or struct into the `target` struct.","ref":"OnePiece.Commanded.Helpers.html#struct_from/2"},{"type":"function","title":"OnePiece.Commanded.Helpers.tracing_from_metadata/1","doc":"Returns a keyword list containing the \"correlation id\" and \"causation id\" tracing.\n\n    iex> OnePiece.Commanded.Helpers.tracing_from_metadata(%{\n    ...>   event_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\",\n    ...>   correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"\n    ...> })\n    ...>\n    [causation_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\", correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"]\n\nUseful when dispatching commands to copy-forward `t:Commanded.Event.Handler.metadata/0` tracing information.\n\n    defmodule MyProcessor do\n        application: MyApp,\n      use Commanded.Event.Handler,\n        name: \"my_processor\"\n\n      alias OnePiece.Commanded.Helpers\n\n      def handle(%MyEvent{} = event, metadata) do\n        MyApp.dispatch(\n          %MyCommand{},\n          # copy-forward the information\n          Helpers.tracing_from_metadata(metadata)\n        )\n      end\n    end","ref":"OnePiece.Commanded.Helpers.html#tracing_from_metadata/1"},{"type":"function","title":"OnePiece.Commanded.Helpers.tracing_from_metadata/2","doc":"Adds the \"correlation id\" and \"causation id\" tracing to an existing keyword list configuration option.\n\n    iex> OnePiece.Commanded.Helpers.tracing_from_metadata([timeout: 30_000], %{\n    ...>   event_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\",\n    ...>   correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"\n    ...> })\n    ...>\n    [timeout: 30_000, causation_id: \"26eb06fe-9ba6-4f58-a2dd-2bdba73de4f2\", correlation_id: \"f634ba94-145c-4fa7-bf7f-0d73dd83b446\"]\n\nUseful when dispatching commands to copy-forward the `t:Commanded.Event.Handler.metadata/0` tracing information and\nwants to also add other keyword list options.\n\n    defmodule MyProcessor do\n      use Commanded.Event.Handler,\n        application: MyApp,\n        name: \"my_processor\"\n\n      alias OnePiece.Commanded.Helpers\n\n      def handle(%MyEvent{} = event, metadata) do\n        MyApp.dispatch(\n          %MyCommand{},\n          # copy-forward the information\n          Helpers.tracing_from_metadata([timeout: 30_000], metadata)\n        )\n      end\n    end","ref":"OnePiece.Commanded.Helpers.html#tracing_from_metadata/2"},{"type":"behaviour","title":"OnePiece.Commanded.Id","doc":"A module for dealing IDs.","ref":"OnePiece.Commanded.Id.html"},{"type":"function","title":"OnePiece.Commanded.Id.new/0","doc":"Generates a UUID using the version 4 scheme, as described in\n[RFC 4122](https://datatracker.ietf.org/doc/html/rfc4122).\n\n    iex> OnePiece.Commanded.Id.new() |> String.length()\n    36","ref":"OnePiece.Commanded.Id.html#new/0"},{"type":"callback","title":"OnePiece.Commanded.Id.new/0","doc":"Generates a new ID.","ref":"OnePiece.Commanded.Id.html#c:new/0"},{"type":"protocol","title":"OnePiece.Commanded.ProtobufMapper","doc":"Protocol for encoding and decoding `Protobuf` structs.\n\nThe protocol is required when using the `OnePiece.Commanded.EventStore.ProtobufJsonbSerializer` serializer and\nworking with Aggregate snapshotting. Since the Aggregates are not `Protobuf` structs, the serializer needs to\nencode and decode them to a `Protobuf` struct.","ref":"OnePiece.Commanded.ProtobufMapper.html"},{"type":"protocol","title":"Example - OnePiece.Commanded.ProtobufMapper","doc":"```elixir\ndefmodule MyApp.MyAggregate do\n  alias MyApp.Proto.MyAggregateProto\n\n  use OnePiece.Commanded.Aggregate, identifier: :id\n\n  embedded_schema do\n    field(:initiated?, :boolean, default: false)\n  end\nend\n\n\ndefimpl OnePiece.Commanded.ProtobufMapper, for: MyApp.MyAggregate do\n  alias MyApp.MyAggregate\n  alias MyApp.Proto.MyAggregateProto\n\n  def message_module(%MyAggregate{} = _aggregate), do: MyAggregateProto\n\n  def to_proto_message(%MyAggregate{} = aggregate) do\n    %MyAggregateProto{\n      id: aggregate.id,\n      initiated: aggregate.initiated?\n    }\n  end\n\n  def from_proto_message(%MyAggregate{} = aggregate, %MyAggregateProto{} = proto_message) do\n    aggregate\n    |> Map.put(:id, proto_message.id)\n    |> Map.put(:initiated?, proto_message.initiated)\n  end\nend\n```","ref":"OnePiece.Commanded.ProtobufMapper.html#module-example"},{"type":"function","title":"OnePiece.Commanded.ProtobufMapper.from_proto_message/2","doc":"Given the initial message, and the decoded `Protobuf` struct, returns the decoded term.\n\nThe default implementation returns the decoded term itself because implicitly expects the initial message to be a\n`Protobuf` struct. No need to map the decoded term to the initial message.","ref":"OnePiece.Commanded.ProtobufMapper.html#from_proto_message/2"},{"type":"function","title":"OnePiece.Commanded.ProtobufMapper.proto_module/1","doc":"Returns the module name of the `Protobuf` struct.\n\nThe default implementation returns the struct name of the message because implicitly expects the struct to be a\n`Protobuf` struct. No need to map the struct to the message.","ref":"OnePiece.Commanded.ProtobufMapper.html#proto_module/1"},{"type":"type","title":"OnePiece.Commanded.ProtobufMapper.t/0","doc":"All the types that implement this protocol.","ref":"OnePiece.Commanded.ProtobufMapper.html#t:t/0"},{"type":"function","title":"OnePiece.Commanded.ProtobufMapper.to_proto_message/1","doc":"Given a struct, returns a `Protobuf` struct.\n\nThe default implementation returns the message itself because implicitly expects the struct to be a\n`Protobuf` struct. No need to map the struct to the message.","ref":"OnePiece.Commanded.ProtobufMapper.html#to_proto_message/1"},{"type":"behaviour","title":"OnePiece.Commanded.QueryHandler","doc":"Defines a module as a \"Query Handler\". For more information about queries, please read the following:\n\n- [CQRS pattern](https://docs.microsoft.com/en-us/azure/architecture/patterns/cqrs)","ref":"OnePiece.Commanded.QueryHandler.html"},{"type":"macro","title":"OnePiece.Commanded.QueryHandler.__using__/1","doc":"Convert the module into a `OnePiece.Commanded.QueryHandler`.","ref":"OnePiece.Commanded.QueryHandler.html#__using__/1"},{"type":"macro","title":"Usage - OnePiece.Commanded.QueryHandler.__using__/1","doc":"defmodule MyQueryHandler do\n      use OnePiece.Commanded.QueryHandler\n      import Ecto.Query, only: [from: 2]\n\n      @impl OnePiece.Commanded.QueryHandler\n      def handle(params) do\n        query = from u in User,\n                  where: u.age > 18 or is_nil(params.email),\n                  select: u\n\n        {:ok, Repo.all(query)}\n      end\n    end","ref":"OnePiece.Commanded.QueryHandler.html#__using__/1-usage"},{"type":"type","title":"OnePiece.Commanded.QueryHandler.error/0","doc":"","ref":"OnePiece.Commanded.QueryHandler.html#t:error/0"},{"type":"callback","title":"OnePiece.Commanded.QueryHandler.handle/1","doc":"Handle the incoming `params` and return the result data.","ref":"OnePiece.Commanded.QueryHandler.html#c:handle/1"},{"type":"type","title":"OnePiece.Commanded.QueryHandler.view_model/0","doc":"","ref":"OnePiece.Commanded.QueryHandler.html#t:view_model/0"},{"type":"module","title":"OnePiece.Commanded.TypeProvider","doc":"Implements `Commanded.EventStore.TypeProvider` behavior. Using macros to generate the behavior.","ref":"OnePiece.Commanded.TypeProvider.html"},{"type":"macro","title":"OnePiece.Commanded.TypeProvider.import_type_provider/1","doc":"Imports all the types from another module defined by `OnePiece.Commanded.TypeProvider`.","ref":"OnePiece.Commanded.TypeProvider.html#import_type_provider/1"},{"type":"macro","title":"Example - OnePiece.Commanded.TypeProvider.import_type_provider/1","doc":"defmodule UserTypeProvider do\n      use OnePiece.Commanded.TypeProvider\n      # ...\n    end\n\n    defmodule MyAppTypeProvider do\n      use OnePiece.Commanded.TypeProvider\n\n      import_type_provider UserTypeProvider\n    end","ref":"OnePiece.Commanded.TypeProvider.html#import_type_provider/1-example"},{"type":"macro","title":"OnePiece.Commanded.TypeProvider.register_type/2","doc":"Registers a mapping from an name string to an Elixir Module that defines a struct.","ref":"OnePiece.Commanded.TypeProvider.html#register_type/2"},{"type":"macro","title":"Example - OnePiece.Commanded.TypeProvider.register_type/2","doc":"defmodule MyTypeProvider do\n      use OnePiece.Commanded.TypeProvider,\n        prefix: \"accounts.\" # optional, adds the prefix to the type name\n\n      register_type \"account_created\", AccountCreated\n    end","ref":"OnePiece.Commanded.TypeProvider.html#register_type/2-example"},{"type":"behaviour","title":"OnePiece.Commanded.Aggregate","doc":"Defines \"Aggregate\" modules.","ref":"OnePiece.Commanded.Aggregate.html"},{"type":"macro","title":"OnePiece.Commanded.Aggregate.__using__/1","doc":"Convert the module into a `Aggregate` behaviour and a `t:t/0`.\n\nIt adds an `apply/2` callback to the module as a fallback, return the aggregate as it is.","ref":"OnePiece.Commanded.Aggregate.html#__using__/1"},{"type":"macro","title":"Options - OnePiece.Commanded.Aggregate.__using__/1","doc":"- `:identifier` - The aggregate identifier key.\n- `:identity_prefix` (optional) - The prefix to be used for the identity.","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-options"},{"type":"macro","title":"Identifier - OnePiece.Commanded.Aggregate.__using__/1","doc":"The `identifier` is used to identify the aggregate. It uses the `@primary_key` attribute to define the column and type.\n\n> #### Schema Field Registration {: .info}\n> `identifier` is automatically registered as a field in the `embedded_schema`.\n> Do not define the field in the `embedded_schema` yourself again.","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-identifier"},{"type":"macro","title":"Using - OnePiece.Commanded.Aggregate.__using__/1","doc":"- `OnePiece.Commanded.Entity`","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-using"},{"type":"macro","title":"Usage - OnePiece.Commanded.Aggregate.__using__/1","doc":"defmodule Account do\n      use OnePiece.Commanded.Aggregate, identifier: :name\n\n      embedded_schema do\n        field :description, :string\n      end\n\n      @impl OnePiece.Commanded.Aggregate\n      def apply(%Account{} = aggregate, %AccountOpened{} = event) do\n        aggregate\n        |> Map.put(:name, event.name)\n        |> Map.put(:description, event.description)\n      end\n    end","ref":"OnePiece.Commanded.Aggregate.html#__using__/1-usage"},{"type":"callback","title":"OnePiece.Commanded.Aggregate.apply/2","doc":"Apply a given event to the aggregate returning the new aggregate state.","ref":"OnePiece.Commanded.Aggregate.html#c:apply/2"},{"type":"callback","title":"Example - OnePiece.Commanded.Aggregate.apply/2","doc":"def apply(%MyAggregate{} = aggregate, %MyEvent{} = event) do\n      aggregate\n      |> Map.put(:name, event.name)\n      |> Map.put(:description, event.description)\n    end","ref":"OnePiece.Commanded.Aggregate.html#c:apply/2-example"},{"type":"type","title":"OnePiece.Commanded.Aggregate.event/0","doc":"","ref":"OnePiece.Commanded.Aggregate.html#t:event/0"},{"type":"type","title":"OnePiece.Commanded.Aggregate.t/0","doc":"A struct that represents an aggregate.","ref":"OnePiece.Commanded.Aggregate.html#t:t/0"},{"type":"module","title":"OnePiece.Commanded.Command","doc":"Defines \"Command\" modules.","ref":"OnePiece.Commanded.Command.html"},{"type":"macro","title":"OnePiece.Commanded.Command.__using__/1","doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Command.html#__using__/1"},{"type":"macro","title":"Options - OnePiece.Commanded.Command.__using__/1","doc":"- `:aggregate_identifier` - The aggregate identifier key.\n- `:identity_prefix` (optional) - The prefix to be used for the identity.","ref":"OnePiece.Commanded.Command.html#__using__/1-options"},{"type":"macro","title":"Aggregate Identifier - OnePiece.Commanded.Command.__using__/1","doc":"The `aggregate_identifier` is used to identify the aggregate that the command is associated with. It uses the\n`@primary_key` attribute to define the column and type.\n\n> #### Schema Field Registration {: .info}\n> `aggregate_identifier` is automatically registered as a field in the `embedded_schema`.\n> Do not define the field in the `embedded_schema` yourself again.","ref":"OnePiece.Commanded.Command.html#__using__/1-aggregate-identifier"},{"type":"macro","title":"Using - OnePiece.Commanded.Command.__using__/1","doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Command.html#__using__/1-using"},{"type":"macro","title":"Usage - OnePiece.Commanded.Command.__using__/1","doc":"defmodule MyCommand do\n      use OnePiece.Commanded.Command, aggregate_identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the aggregate identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule AssignRole do\n      use OnePiece.Commanded.Command,\n        aggregate_identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Command.html#__using__/1-usage"},{"type":"type","title":"OnePiece.Commanded.Command.aggregate_identifier_opt/0","doc":"The aggregate identifier key of the event.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Command.html#t:aggregate_identifier_opt/0"},{"type":"type","title":"OnePiece.Commanded.Command.t/0","doc":"A struct that represents a command.","ref":"OnePiece.Commanded.Command.html#t:t/0"},{"type":"module","title":"OnePiece.Commanded.Entity","doc":"Defines \"Entity\" modules.","ref":"OnePiece.Commanded.Entity.html"},{"type":"macro","title":"OnePiece.Commanded.Entity.__using__/1","doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Entity.html#__using__/1"},{"type":"macro","title":"Identifier - OnePiece.Commanded.Entity.__using__/1","doc":"The `identifier` is used to identify the entity. It uses the `@primary_key` attribute to define the column and type.\n\n> #### Schema Field Registration {: .info}\n> `identifier` is automatically registered as a field in the `embedded_schema`.\n> Do not define the field in the `embedded_schema` yourself again.","ref":"OnePiece.Commanded.Entity.html#__using__/1-identifier"},{"type":"macro","title":"Using - OnePiece.Commanded.Entity.__using__/1","doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Entity.html#__using__/1-using"},{"type":"macro","title":"Usage - OnePiece.Commanded.Entity.__using__/1","doc":"defmodule MyEntity do\n      use OnePiece.Commanded.Entity, identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule IdentityRole do\n      use OnePiece.Commanded.Entity,\n        identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Entity.html#__using__/1-usage"},{"type":"type","title":"OnePiece.Commanded.Entity.identifier_opt/0","doc":"The identity key of the entity.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Entity.html#t:identifier_opt/0"},{"type":"type","title":"OnePiece.Commanded.Entity.identity/0","doc":"The identity of an entity.","ref":"OnePiece.Commanded.Entity.html#t:identity/0"},{"type":"type","title":"OnePiece.Commanded.Entity.t/0","doc":"A struct that represents an entity.","ref":"OnePiece.Commanded.Entity.html#t:t/0"},{"type":"type","title":"OnePiece.Commanded.Entity.using_opts/0","doc":"","ref":"OnePiece.Commanded.Entity.html#t:using_opts/0"},{"type":"module","title":"OnePiece.Commanded.Enum","doc":"Defines an Enum type module.","ref":"OnePiece.Commanded.Enum.html"},{"type":"macro","title":"OnePiece.Commanded.Enum.__using__/1","doc":"Converts the module into a struct with an `:value` enum field.","ref":"OnePiece.Commanded.Enum.html#__using__/1"},{"type":"macro","title":"Using - OnePiece.Commanded.Enum.__using__/1","doc":"- `Ecto.Schema`\n- `Ecto.Type`","ref":"OnePiece.Commanded.Enum.html#__using__/1-using"},{"type":"macro","title":"Derives - OnePiece.Commanded.Enum.__using__/1","doc":"- `Jason.Encoder`","ref":"OnePiece.Commanded.Enum.html#__using__/1-derives"},{"type":"macro","title":"Usage - OnePiece.Commanded.Enum.__using__/1","doc":"defmodule BankAccountType do\n      use OnePiece.Commanded.Enum, values: [:business, :personal]\n    end\n\n    {:ok, type} = BankAccountType.new(:business)\n\nYou can use it in your `Ecto.Schema` like this:\n\n    defmodule BankAccount do\n      use Ecto.Schema\n\n      embedded_schema do\n        field :type, BankAccountType\n      end\n    end","ref":"OnePiece.Commanded.Enum.html#__using__/1-usage"},{"type":"module","title":"OnePiece.Commanded.Event","doc":"Defines \"Event\" modules.","ref":"OnePiece.Commanded.Event.html"},{"type":"macro","title":"OnePiece.Commanded.Event.__using__/1","doc":"Converts the module into an `t:t/0`.","ref":"OnePiece.Commanded.Event.html#__using__/1"},{"type":"macro","title":"Options - OnePiece.Commanded.Event.__using__/1","doc":"- `:aggregate_identifier` - The aggregate identifier key.","ref":"OnePiece.Commanded.Event.html#__using__/1-options"},{"type":"macro","title":"Aggregate Identifier - OnePiece.Commanded.Event.__using__/1","doc":"The `aggregate_identifier` is used to identify the aggregate that the event is associated with. It uses the\n`@primary_key` attribute to define the column and type.\n\n> #### Schema Field Registration {: .info}\n> `aggregate_identifier` is automatically registered as a field in the `embedded_schema`.\n> Do not define the field in the `embedded_schema` yourself again.","ref":"OnePiece.Commanded.Event.html#__using__/1-aggregate-identifier"},{"type":"macro","title":"Using - OnePiece.Commanded.Event.__using__/1","doc":"- `OnePiece.Commanded.ValueObject`","ref":"OnePiece.Commanded.Event.html#__using__/1-using"},{"type":"macro","title":"Usage - OnePiece.Commanded.Event.__using__/1","doc":"defmodule MyEvent do\n      use OnePiece.Commanded.Event, aggregate_identifier: :id\n\n      embedded_schema do\n        # ...\n      end\n    end\n\nYou can also define a custom type as the aggregate identifier:\n\n    defmodule IdentityRoleId do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :identity_id, :string\n        field :role_id, :string\n      end\n    end\n\n    defmodule IdentityRoleAssigned do\n      use OnePiece.Commanded.Event,\n        aggregate_identifier: {:id, IdentityRoleId}\n\n      embedded_schema do\n        # ...\n      end\n    end","ref":"OnePiece.Commanded.Event.html#__using__/1-usage"},{"type":"type","title":"OnePiece.Commanded.Event.aggregate_identifier_opt/0","doc":"The aggregate identifier key of the event.\n\nIf it's a tuple, the type must be a module that implements the `OnePiece.Commanded.ValueObject` module or [`Ecto` built-in types](https://hexdocs.pm/ecto/Ecto.Schema.html#module-types-and-casting)","ref":"OnePiece.Commanded.Event.html#t:aggregate_identifier_opt/0"},{"type":"type","title":"OnePiece.Commanded.Event.t/0","doc":"A struct that represents an event.","ref":"OnePiece.Commanded.Event.html#t:t/0"},{"type":"module","title":"OnePiece.Commanded.ValueObject","doc":"Defines \"Value Object\" modules.","ref":"OnePiece.Commanded.ValueObject.html"},{"type":"macro","title":"OnePiece.Commanded.ValueObject.__using__/1","doc":"Converts the module into an `Ecto.Schema` and add factory functions to create structs.","ref":"OnePiece.Commanded.ValueObject.html#__using__/1"},{"type":"macro","title":"Using - OnePiece.Commanded.ValueObject.__using__/1","doc":"- `Ecto.Schema`\n- `Ecto.Type`","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-using"},{"type":"macro","title":"Derives - OnePiece.Commanded.ValueObject.__using__/1","doc":"- `Jason.Encoder`","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-derives"},{"type":"macro","title":"Usage - OnePiece.Commanded.ValueObject.__using__/1","doc":"defmodule MyValueObject do\n      use OnePiece.Commanded.ValueObject\n\n      embedded_schema do\n        field :title, :string\n        # ...\n      end\n    end\n\n    {:ok, my_value} = MyValueObject.new(%{title: \"Hello, World!\"})","ref":"OnePiece.Commanded.ValueObject.html#__using__/1-usage"},{"type":"function","title":"OnePiece.Commanded.ValueObject.changeset/2","doc":"Returns an `t:Ecto.Changeset.t/0` for a given value object struct.\n\nIt reads the `@enforced_keys` from the struct and validates the required\nfields. Also, it casts the embeds. It is useful when you override the\n`changeset/2` function in your value object.","ref":"OnePiece.Commanded.ValueObject.html#changeset/2"},{"type":"function","title":"Examples - OnePiece.Commanded.ValueObject.changeset/2","doc":"```elixir\ndefmodule MyValueObject do\n  use OnePiece.Commanded.ValueObject\n\n  @enforce_keys [:title, :amount]\n  embedded_schema do\n    field :title, :string\n    field :amount, :integer\n  end\n\n  def changeset(message, attrs) do\n    message\n    |> ValueObject.changeset(attrs)\n    |> Changeset.validate_number(:amount, greater_than: 0)\n  end\nend\n```","ref":"OnePiece.Commanded.ValueObject.html#changeset/2-examples"},{"type":"module","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase","doc":"This module helps with test cases for testing aggregate states, and command handlers.","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html"},{"type":"module","title":"Options - OnePiece.Commanded.TestSupport.CommandHandlerCase","doc":"- `aggregate`: The aggregate module to use in the command handler case. If not provided, then you must provide a\n  `handler` option.\n- `handler`: The command handler module to use in the command handler case. If the `aggregate` option is not provided,\n  then the `Handler.Aggregate` module will be used as the aggregate module. Read more about transaction script\n  command handler at `OnePiece.Commanded.CommandRouter.register_transaction_script/2`.","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#module-options"},{"type":"module","title":"Usage - OnePiece.Commanded.TestSupport.CommandHandlerCase","doc":"After import the test support file, you should be able to have the module in your test files.\n\n    defmodule MyAggregateTest do\n      use OnePiece.Commanded.TestSupport.CommandHandlerCase,\n        handler: MyCommandHandler,\n        async: true\n\n      describe \"my aggregate\" do\n        test \"should do something\" do\n          assert_events(\n            [%InitialEvent{}],\n            %DoSomething{},\n            [%SomethingHappened{}]\n          )\n        end\n\n        test \"the state\" do\n          assert_state(\n            [%InitialEvent{}],\n            %DoSomething{},\n            %MyAggregate{}\n          )\n        end\n\n        test \"the error\" do\n          assert_error(\n            [%InitialEvent{}],\n            %DoSomething{},\n            :already_exists\n          )\n        end\n      end\n    end","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#module-usage"},{"type":"function","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_error/5","doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_error/5"},{"type":"function","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_events/5","doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_events/5"},{"type":"function","title":"OnePiece.Commanded.TestSupport.CommandHandlerCase.assert_state/5","doc":"","ref":"OnePiece.Commanded.TestSupport.CommandHandlerCase.html#assert_state/5"},{"type":"extras","title":"OnePiece.Commanded","doc":"# OnePiece.Commanded\n\nExtend `Commanded` package. A swiss army knife for applications following Domain-Driven Design (DDD), Event Sourcing (ES), and Command and Query Responsibility Segregation (CQRS).","ref":"readme.html"},{"type":"extras","title":"Documentation - OnePiece.Commanded","doc":"","ref":"readme.html#documentation"},{"type":"extras","title":"References - OnePiece.Commanded","doc":"- [API Reference](api-reference.html)","ref":"readme.html#references"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"Unreleased - Changelog","doc":"","ref":"changelog.html#unreleased"},{"type":"extras","title":"v0.25.1 - 2025-02-04 - Changelog","doc":"- Fix order of `Code.ensure_compiled!/1` in `OnePiece.Commanded.CommandRouter` ensure that the aggregate is compiled\n  before the command.","ref":"changelog.html#v0-25-1-2025-02-04"},{"type":"extras","title":"v0.25.0 - 2025-01-31 - Changelog","doc":"- Added `OnePiece.Commanded.ValueObject.validate/2` function to `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-25-0-2025-01-31"},{"type":"extras","title":"v0.24.0 - 2025-01-17 - Changelog","doc":"- Added `OnePiece.Commanded.ValueObject.changeset/2` function.","ref":"changelog.html#v0-24-0-2025-01-17"},{"type":"extras","title":"v0.23.0 - 2025-01-13 - Changelog","doc":"- Added `OnePiece.Commanded.ProtobufMapper` protocol.\n- Extended `OnePiece.Commanded.EventStore.ProtobufJsonbSerializer` to use `OnePiece.Commanded.ProtobufMapper`.","ref":"changelog.html#v0-23-0-2025-01-13"},{"type":"extras","title":"v0.22.0 - 2024-12-08 - Changelog","doc":"- Added `OnePiece.Commanded.EventStore.ProtobufJsonbSerializer` module.","ref":"changelog.html#v0-22-0-2024-12-08"},{"type":"extras","title":"v0.21.4 - 2024-12-06 - Changelog","doc":"- Added `generated: true` to `quote`.","ref":"changelog.html#v0-21-4-2024-12-06"},{"type":"extras","title":"v0.21.3 - 2024-11-27 - Changelog","doc":"- Added support for casting maps in `OnePiece.Commanded.Enum`.","ref":"changelog.html#v0-21-3-2024-11-27"},{"type":"extras","title":"v0.21.2 - 2024-11-26 - Changelog","doc":"- Added support for transaction script command handler to `OnePiece.Commanded.TestSupport.CommandHandlerCase`.","ref":"changelog.html#v0-21-2-2024-11-26"},{"type":"extras","title":"v0.21.1 - 2024-11-26 - Changelog","doc":"- Fix casting structs in `OnePiece.Commanded.ValueObject`. Related to https://github.com/elixir-ecto/ecto/issues/4168","ref":"changelog.html#v0-21-1-2024-11-26"},{"type":"extras","title":"v0.21.0 - 2024-11-02 - Changelog","doc":"- Added `OnePiece.Commanded.Enum` module.","ref":"changelog.html#v0-21-0-2024-11-02"},{"type":"extras","title":"v0.20.0 - 2024-05-05 - Changelog","doc":"- Added `OnePiece.Commanded.CommandRouter` module.","ref":"changelog.html#v0-20-0-2024-05-05"},{"type":"extras","title":"v0.19.1 - 2023-10-04 - Changelog","doc":"- Fixed `new/1` typespec of `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-19-1-2023-10-04"},{"type":"extras","title":"v0.19.0 - 2023-06-15 - Changelog","doc":"`OnePiece.Commanded.ValueObject` implements `Ecto.Type`, it means that `cast/2`, `dump/2`, and `load/2` are\nadded to every module that uses `OnePiece.Commanded.ValueObject`. This will allow you to use\n`OnePiece.Commanded.ValueObject` as a field in your `Ecto.Schema` and as custom type in your\n`OnePiece.Commanded.Aggregate`, `OnePiece.Commanded.Command`, and `OnePiece.Commanded.Event`.\n\nAdded support for custom types for `OnePiece.Commanded.Aggregate`, `OnePiece.Commanded.Command`, and\n`OnePiece.Commanded.Event`. This will allow you to have Custom aggregate identity. Read more about at\nhttps://hexdocs.pm/commanded/commands.html#define-aggregate-identity under \"Custom aggregate identity\".\n\n```elixir\ndefmodule AccountNumber do\n  use OnePiece.Commanded.ValueObject\n\n  embedded_schema do\n    field :account_number, :string\n    field :branch, :string\n  end\n\n  # You must implement `String.Chars` protocol in order to work when dispatching the Command.\n  defimpl String.Chars do\n    def to_string(%AccountNumber{branch: branch, account_number: account_number}) do\n      branch <> \":\" <> account_number\n    end\n  end\nend\n\ndefmodule DepositAccount do\n  use OnePiece.Commanded.Aggregate,\n    identifier: {:account_number, AccountNumber}\n\n  embedded_schema do\n    # ...\n  end\nend\n\ndefmodule DepositAccountOpened do\n  use OnePiece.Commanded.Event,\n    aggregate_identifier: {:account_number, AccountNumber}\n\n  embedded_schema do\n    # ...\n  end\nend\n\ndefmodule OpenDepositAccount do\n  use OnePiece.Commanded.Command,\n    aggregate_identifier: {:account_number, AccountNumber}\n\n  embedded_schema do\n    # ...\n  end\nend\n```","ref":"changelog.html#v0-19-0-2023-06-15"},{"type":"extras","title":"v0.18.0 - 2023-05-15 - Changelog","doc":"- Added `OnePiece.Commanded.ignore_error/2`.","ref":"changelog.html#v0-18-0-2023-05-15"},{"type":"extras","title":"v0.17.0 - 2023-04-28 - Changelog","doc":"- Added `OnePiece.Commanded.Aggregate.StatelessLifespan` module.","ref":"changelog.html#v0-17-0-2023-04-28"},{"type":"extras","title":"v0.16.0 - 2023-04-11 - Changelog","doc":"- Added `OnePiece.Commanded.EventStore.JsonbSerializer` module.","ref":"changelog.html#v0-16-0-2023-04-11"},{"type":"extras","title":"v0.15.2 - 2023-03-14 - Changelog","doc":"- Fix casting already cast structs in `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-15-2-2023-03-14"},{"type":"extras","title":"v0.15.1 - 2022-12-24 - Changelog","doc":"- Fix `OnePiece.Commanded.Event` and `OnePiece.Commanded.Entity` to extend from `OnePiece.Commanded.ValueObject`.","ref":"changelog.html#v0-15-1-2022-12-24"},{"type":"extras","title":"v0.15.0 - 2022-12-23 - Changelog","doc":"- Added overridable to `OnePiece.Commanded.ValueObject.new/1`\n- Added overridable to `OnePiece.Commanded.ValueObject.new!/1`\n- Added overridable to `OnePiece.Commanded.ValueObject.changeset/2`","ref":"changelog.html#v0-15-0-2022-12-23"},{"type":"extras","title":"v0.14.0 - 2022-09-4 - Changelog","doc":"- Added `OnePiece.Commanded.increase_failure_counter/1`","ref":"changelog.html#v0-14-0-2022-09-4"},{"type":"extras","title":"v0.13.0 - 2022-08-19 - Changelog","doc":"- Added `OnePiece.Commanded.tracing_from_metadata/1`.\n- Added `OnePiece.Commanded.tracing_from_metadata/2`.","ref":"changelog.html#v0-13-0-2022-08-19"},{"type":"extras","title":"v0.12.0 - 2022-08-19 - Changelog","doc":"- Added `OnePiece.Commanded.skip_or_retry/2`.\n- Added `OnePiece.Commanded.skip_or_retry/3`.\n- Added `OnePiece.Commanded.Helpers.skip_or_retry/3`.\n- Added `OnePiece.Commanded.Helpers.skip_or_retry/3`.\n- Undo deprecation of `OnePiece.Commanded.cast_to/3`.","ref":"changelog.html#v0-12-0-2022-08-19"},{"type":"extras","title":"v0.11.1 - 2022-07-21 - Changelog","doc":"- Fix register type with a struct with enforce keys in `OnePiece.Commanded.TypeProvider` module.","ref":"changelog.html#v0-11-1-2022-07-21"},{"type":"extras","title":"v0.11.0 - 2022-07-1 - Changelog","doc":"- Add `OnePiece.Commanded.Helpers.tracing_from_metadata/1`\n- Add `OnePiece.Commanded.Helpers.tracing_from_metadata/2`","ref":"changelog.html#v0-11-0-2022-07-1"},{"type":"extras","title":"v0.10.1 - 2022-06-28 - Changelog","doc":"- Deprecate `OnePiece.Commanded.cast_to/3` (wrong module) and delegate to\n  `OnePiece.Commanded.Helpers` instead.\n- Add `OnePiece.Commanded.Helpers.cast_to/3`","ref":"changelog.html#v0-10-1-2022-06-28"},{"type":"extras","title":"v0.10.0 - 2022-06-23 - Changelog","doc":"- Add `OnePiece.Commanded.cast_to/3`","ref":"changelog.html#v0-10-0-2022-06-23"},{"type":"extras","title":"v0.9.1 - 2022-06-22 - Changelog","doc":"- Move test support under `lib` directory.","ref":"changelog.html#v0-9-1-2022-06-22"},{"type":"extras","title":"v0.7.0 - 2022-02-14 - Changelog","doc":"- Added `OnePiece.Commanded.Id`\n- Deprecated `OnePiece.Commanded.Helpers.generate_uuid/0`. Use\n  `OnePiece.Commanded.Id.new/0` instead.","ref":"changelog.html#v0-7-0-2022-02-14"},{"type":"extras","title":"v0.6.0 - 2022-02-14 - Changelog","doc":"- Added `prefix` option to using `OnePiece.Commanded.TypeProvider`\n\n  ```elixir\n  defmodule MyTypeProvider do\n    use OnePiece.Commanded.TypeProvider, prefix: \"accounts.\"\n  end\n  ```","ref":"changelog.html#v0-6-0-2022-02-14"},{"type":"extras","title":"v0.5.0 - 2022-02-14 - Changelog","doc":"- Added `OnePiece.Commanded.TypeProvider`","ref":"changelog.html#v0-5-0-2022-02-14"},{"type":"extras","title":"v0.4.0 - 2021-11-25 - Changelog","doc":"- Improve documentation","ref":"changelog.html#v0-4-0-2021-11-25"},{"type":"extras","title":"v0.4.0 - 2021-08-15 - Changelog","doc":"- Add `OnePiece.Commanded.Event` module","ref":"changelog.html#v0-4-0-2021-08-15"},{"type":"extras","title":"v0.3.0 - 2021-08-15 - Changelog","doc":"- Fix docs\n- Fix version bump","ref":"changelog.html#v0-3-0-2021-08-15"},{"type":"extras","title":"v0.2.1 - 2021-08-15 - Changelog","doc":"- Add test support files","ref":"changelog.html#v0-2-1-2021-08-15"},{"type":"extras","title":"v0.2.0 - 2021-08-08 - Changelog","doc":"- Fix putting aggregate id to command function name","ref":"changelog.html#v0-2-0-2021-08-08"},{"type":"extras","title":"v0.1.3 - 2021-08-08 - Changelog","doc":"- Fix aliases","ref":"changelog.html#v0-1-3-2021-08-08"},{"type":"extras","title":"v0.1.2 - 2021-08-08 - Changelog","doc":"- Fix `__using__` for aggregate, command handler, query handler, and value\n  object","ref":"changelog.html#v0-1-2-2021-08-08"},{"type":"extras","title":"v0.1.1 - 2021-08-08 - Changelog","doc":"- Remove dead-code","ref":"changelog.html#v0-1-1-2021-08-08"},{"type":"extras","title":"v0.1.0 - 2021-08-08 - Changelog","doc":"- Initial release","ref":"changelog.html#v0-1-0-2021-08-08"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}